
#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <mem.h>
#include <conio.h>

#include "com.h"
#include "wait.h"
#include "cgdk.h"
#include "timer.h"
#include "comirq.h"

#ifdef __TURBOC__

void KEYB_update (void)
{

}

#else

#include "keyb.h"

#endif

int port, irq, mode;
long waitPhase;

int waitHandler (void)
{
  int c;

  WAIT_flashPrint (waitPhase, MAXLONG);
  waitPhase++;
  while (kbhit ())
  {
    c = getch ();
    if (c == 27)
    {
      return 1;
    }
  }
  return 0;
}


#define PEER 0
#define MASTER 1
#define SLAVE 2

static char *modeDesc [] = {"Peer to peer", "Master", "Slave"};

static int login (int mode)
{
  int result;

  waitPhase = 0;

  printf ("\nWaiting for the remote host to connect ...\n");

  switch (mode)
  {
    case PEER:
    {
      result = COM_login ("", waitHandler);
      break;
    }
    case MASTER:
    {
      result = COM_create ("", waitHandler);
      break;
    }
    case SLAVE:
    {
      result = COM_join ("", waitHandler);
      break;
    }
  }

  printf ("\r \n");

  return result;
}

#define COM_BUFSIZE 512

static int listen (char *buf)
{
  while (COM_receive (buf))
  {
    printf ("%s: %s\n", COM_user == COM_USER_PEER ? "Remote" : COM_remoteName, buf);
  }

  switch (COM_command)
  {
    case COM_COMMAND_LOGOUT:
    {
      printf ("\nConnection has been terminated by the remote host.\n");
      return 1;
    }
  }

  return 0;
}

static void talk (void)
{
  char s [COM_BUFSIZE];

  printf ("\n");

  if (COM_user)
  {
    while (listen (s) == 0)
    {
       printf ("%s: ", COM_user == COM_USER_PEER ? "Local" : COM_localName);
       gets (s);
       if (strlen (s))
       {
         COM_send (s, strlen (s)+1);
       }
       else
       {
         listen (s);
         break;
       }
    }
  }
  else
  {
    printf ("You are not connected.\n");
  }
}

static void printSetupInfo (void)
{
  printf ("Serial port number   : %i\n"
          "Serial port address  : 0%Xh\n"
          "IRQ number           : %i\n"
          "IRQ interrupt number : %X\n"
          "Communication speed  : %li bauds (%.2f kB/s)\n"
          "Mode                 : %s\n",
          COM_port, COM_addr, COM_irq, COM_int,
          COM_baudSpeed, (float)COM_byteSpeed/1024,
          modeDesc [mode]);
}

void help (void)
{
  printf ("\n"
          "Syntax is: COM (serial_port_number) [(irq_number) [/M|/S]]\n" // (output_port)\n"
          "           serial_port_number - 1, 2, 3 or 4\n"
          "           irq_number         - irq number from 0 to 7, or -1 to autodetect\n"
          "           /M                 - connect as master\n"
          "           /S                 - connect as slave\n\n");

  CGDK_exit ("", EXIT_FAILURE);
}

static void printInfo (void)
{
}


static void processCommandLine (void)
{
  switch (_argc)
  {
    case 1+1:
      sscanf (_argv [1], "%i", &port);
      irq = COM_INTERRUPT_AUTODETECT;
      mode = PEER;
      break;
    case 1+2:
    {
      sscanf (_argv [1], "%i", &port);
      sscanf (_argv [2], "%i", &irq);
      mode = PEER;
      break;
    }
    case 1+3:
    {
      sscanf (_argv [1], "%i", &port);
      sscanf (_argv [2], "%i", &irq);
      if (_argv [3] [0] == '/')
        _argv [3] [0] = '-';
      if (strcmpi (_argv [3], "-m")==0)
      {
        mode = MASTER;
      }
      else
      {
        if (strcmpi (_argv [3], "-s")==0)
        {
          mode = SLAVE;
        }
        else
        {
          help ();
        }
      }
      break;
    }
    default:
    {
      help ();
    }
  }
}



int main (void)
{
  printf ("\n"
          "COM - serial communication driver test program\n"
          "\n%s\n", CGDK_copyright);

  #ifndef __TURBOC__
    CGDK_init ();
  #else
    COM_init ();
  #endif

  TIMER_init (TIMER_OLDFREQ_DEF);

  processCommandLine ();

  switch (COM_setup (port, irq, COM_9600BAUDS))
  {
    case COM_INITERR_PORT:
      printf ("Invalid COM port number.\n");
      CGDK_exit ("", EXIT_FAILURE);
      break;
    case COM_INITERR_IRQ:
      printf ("Invalid IRQ number.\n");
      CGDK_exit ("", EXIT_FAILURE);
      break;
  }

  printSetupInfo ();

  switch (login (mode))
  {
    case 0:
      printf ("You are connected to %s.\n", COM_remoteName);
      talk ();
      break;
    default:
      printf ("Connection canceled.\n");
      break;
  }

  COM_logout ();

  printf ("\n"
          "Total bytes sent     : %5li, %5li using 16-bit handler.\n", COM_buf->bytesSent, COM_buf->bytesSent16);
  printf ("Total bytes received : %5li, %5li using 16-bit handler.\n", COM_buf->bytesReceived, COM_buf->bytesReceived16);

  COM_restore ();

  printInfo ();

  TIMER_done ();

  return 0;
}
