
#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <mem.h>

#include "svga.h"
#include "pal.h"
#include "flic.h"
#include "car.h"
#include "cgdk.h"
#include "keyb.h"
#include "tm.h"
#include "timer.h"

#define VIDEOMODE_DEFAULT -1

int main (void)
{
  int i;
  TCAR *carHandle;
  int videoMode;
  int sizeX, sizeY;
  int vsizeX, vsizeY, vbytes, vbits;
  int flicExit;
  int cache;
  char error [256];
  char *carName;
  char *flicName;
  char *s;
  int result;

  printf ("\n"
          "FLIC - The fastest Autodesk FLI/FLC player in the world.\n"
          "\n%s\n",CGDK_copyright);

  CGDK_init ();
  TIMER_init (TIMER_OLDFREQ_DEF);

  KEYB_init ();

  switch (_argc)
  {
    case 1+1:
    {
      cache = 0;
      videoMode = VIDEOMODE_DEFAULT;
      break;
    }
    case 1+2:
    {
      cache = 0;
      videoMode = strtol (_argv [2],NULL,0);
      break;
    }
    case 1+3:
    {
      videoMode = strtol (_argv [2],NULL,0);
      cache = 1;
      break;
    }
    default:
    {
      CGDK_exit ("Syntax is : FLIC <car_filename/flic_filename> [(video_mode)] [cache]\n"
                 "\n"
                 "            car_filename  - Cybertek ARchive containing FLI/FLC file (optional)\n"
                 "            flic_filename - an Autodesk FLI/FLC file\n"
                 "               video mode - number of the video mode to play (optional)\n"
                 "                            for SVGA FLI/FLC, VESA 2.0 video driver required.\n"
                 "                            set to 0 to autodetect video mode (default).\n"
                 "                    cache - enables CAR cache (optional)\n", EXIT_FAILURE);

    }
  }

  if (s = strchr (_argv [1], '/'), s == NULL)
  {
    carName = CAR_EMULATION;
    flicName = _argv [1];
  }
  else
  {
    *s = 0;
    carName = _argv [1];
    flicName = s+1;
  }

  carHandle = CAR_open (carName);
  if (carHandle == NULL)
  {
    sprintf (error, "Invalid CAR file %s.\n", carName);
    CGDK_exit (error, EXIT_FAILURE);
  }

  if (CAR_seek (carHandle, flicName, 0) == -1)
  {
    sprintf (error, "File %s not found.\n", flicName);
    CGDK_exit (error, EXIT_FAILURE);
  }

  if (cache)
  {
    printf ("Loading FLIC file %s into CAR cache ...\n", _argv [1]);
    CAR_loadCache (carHandle, _argv [1]);
  }

  if ((carHandle != NULL) &&
       (FLIC_getInfo (carHandle, flicName, &sizeX, &sizeY) == 0))
  {
    if (videoMode == VIDEOMODE_DEFAULT)
    {
      videoMode = SVGA_recommendVideoMode (sizeX, sizeY, 1, VMODE_LINEAR);
      result = SVGA_getModeInfo (videoMode, &vsizeX, &vsizeY, &vbytes, &vbits);
      if ((result != 0) || (vsizeX < sizeX) || (vsizeY < sizeY))
      {
        CGDK_exit ("No matching video available.\n"
                   "Install a VESA 2.0 compatible video driver and try again.\n", EXIT_FAILURE);
      }
    }
    else
    {
      if (SVGA_getModeInfo (videoMode, &vsizeX, &vsizeY, &vbytes, &vbits) != 0)
        CGDK_exit ("The specified video mode is not supported.\n", EXIT_FAILURE);
      if ((vsizeX < sizeX) || (vsizeY < sizeY))
      {
        CGDK_exit ("The specified video mode is not usable.\n", EXIT_FAILURE);
      }
    }

    SVGA_setVideoMode (videoMode);
    PAL_clear (0,256);

    flicExit = 0;
    do
    {
      if (FLICP_init (carHandle, flicName, 0, 0) == 0)
      {
        for (i = 1; i < 2*FLIC_framesCount; i++)
        {
          FLICP_play ();
          if (KEYB_get () == KEY_ESC)
          {
            flicExit = 1;
            break;
          }
        }
        PAL_fadeOut (FLICP_pal, 100, 0, 256);
        FLICP_done ();
      }
      else
      {
        SVGA_done ();
        sprintf (error,"Error loading FLIC file %s\n", flicName);
        CGDK_exit (error, EXIT_FAILURE);
      }
    }
    while (flicExit==0);
    SVGA_done ();
  }
  else
  {
    sprintf (error,"Invalid FLIC file %s\n", flicName);
    CGDK_exit (error, EXIT_FAILURE);
  }

  return 0;
}
