
#include <stdlib.h>
#include <mem.h>

#include "lzw.h"

#include "cgdk.h"

#define LZW_TABLE_MAXPHRASE 4096 // maximalni pocet frazi ve slovniku

#ifdef __TURBOC__
  #define LZW_TABLE_PLUS 1024
#else
  #define LZW_TABLE_PLUS LZW_TABLE_MAXPHRASE
#endif

#define LZW_TABLE_MAXLEN (LZW_TABLE_MAXPHRASE+LZW_TABLE_PLUS)

//#define LZW_PHRASEDELTA 4
//#define LZW_PHRASEMAX (LZW_TABLE_MAXLEN - LZW_TABLE_MAXLEN/LZW_PHRASEDELTA)

#define NOT_USED -1
#define NONE_PHRASE (2*LZW_TABLE_MAXLEN)
//#define EMPTY -1
#define STACK_EMPTY -1
//#define LZW_TABLE_OVERFLOW -1

int LZW_getCharError (void)
{
  CGDK_exit ("CGDK fatal error: unassigned pointer (int (*LZW_getChar) (void)).\n", EXIT_FAILURE);
  return 0;
}

int LZW_putCharError (int c)
{
  CGDK_exit ("CGDK fatal error: unassigned pointer (int (*LZW_putChar) (int c)).\n",  EXIT_FAILURE);
  return c;
}

static int *LZW_stack; // public stack
static int LZW_sp = 0; // public stack pointer

static int *LZW_cache;


#define LZW_phraseUsed(i) (LZW_phrase[i].prev!=NOT_USED)


inline static int LZW_hashFn (int phrase, int nextValue)
{
  long temp;
  long local;

  local = (phrase + nextValue) | 0x0800;
  temp = local * local;
  #ifdef __TURBOC__
    local = ((temp >> 6) & LZW_TABLE_MAXPHRASE-1) + (temp & (LZW_TABLE_PLUS-1));
  #else
    local = ((temp >> 6) & LZW_TABLE_MAXLEN-1);
  #endif

  return local;
}



static int LZW_putPhrase (TLZW_phrase *LZW_phrase, int phrase, int nextValue)
{
  int index, last;

  index = LZW_hashFn (phrase, nextValue);
  if ( LZW_phraseUsed (index) )
  {
    while (LZW_phrase [index].next != NOT_USED)
    {
      index = LZW_phrase [index].next;
    }

    last = index;

    index += 101;
    do
    {
      index++;
      if ( index >= LZW_TABLE_MAXLEN-1 )
      {
        index = 0;
      }
    }
    while ( LZW_phraseUsed (index) );


    LZW_phrase [last].next = index;
  }

  LZW_phrase [index].value = nextValue;
  LZW_phrase [index].next = NOT_USED;
  LZW_phrase [index].prev = phrase;

  return index;
}





TLZW *LZW_new (void)
{
  TLZW *lzw;

  lzw = (TLZW *) _alloc (sizeof (TLZW));
  if (lzw == NULL)
  {
    return NULL;
  }
  lzw->codeCount = 0;
  lzw->bytesRead = lzw->bytesWritten = 0;
  lzw->putChar = LZW_putCharError;
  lzw->getChar = LZW_getCharError;
  lzw->phrase = (TLZW_phrase *) _alloc (LZW_TABLE_MAXLEN*(sizeof (TLZW_phrase)));
  if (lzw->phrase == NULL)
  {
    _free (lzw);
    return NULL;
  }
  return lzw;
}

void LZW_free (TLZW *lzw)
{
  if (lzw != NULL)
  {
    _free (lzw->phrase);
    _free (lzw);
  }
}

void LZW_init (void)
{
  TLZW *lzw;
  int pos, i;

  if (LZW_stack == NULL)
  {
    LZW_stack = (int *) _alloc (sizeof (int) * LZW_TABLE_MAXLEN);
  }

  if (LZW_stack == NULL)
  {
    CGDK_exit ("Not enough memory for LZW stack.\n", EXIT_FAILURE);
  }

  if (LZW_cache == NULL)
  {
    LZW_cache = (int *) _alloc (sizeof (int) * 256);
  }

  if (LZW_cache == NULL)
  {
    CGDK_exit ("Not enough memory for LZW character lookup buffer.\n", EXIT_FAILURE);
  }

  lzw = LZW_new ();
  if (lzw == NULL)
  {
    CGDK_exit ("Not enough memory for temporary LZW handler.\n", EXIT_FAILURE);
  }

  memset (lzw->phrase, NOT_USED, LZW_TABLE_MAXLEN*sizeof (TLZW_phrase));

  for (i = 0; i <= 255; i++)
  {
    pos = LZW_putPhrase (lzw->phrase, NONE_PHRASE, i);
    LZW_cache [i] = pos;
  }

  LZW_free (lzw);
}

void LZW_done (void)
{
  if (LZW_stack != NULL)
  {
    _free (LZW_stack);
    LZW_stack = NULL;
  }
  if (LZW_cache != NULL)
  {
    _free (LZW_cache);
    LZW_cache = NULL;
  }
}


#include "lzwcode_.cpp"


void LZW_compressInit (TLZW *lzw)
{
  int i, pos;

  // vymazani tabulky frazi (nastaveni na volne)
  memset (lzw->phrase, NOT_USED, LZW_TABLE_MAXLEN*sizeof (TLZW_phrase));

  // slovnik je inicializovan pro vsechny vstupni symboly
  for (i = 0; i <= 255; i++)
  {
    pos = LZW_putPhrase (lzw->phrase, NONE_PHRASE, i);
    lzw->phrase [pos].index = i;
    LZW_cache [i] = pos;
  }
  lzw->codeCount=256;
  lzw->code = NONE_PHRASE; //
  lzw->mode = 0;
}

void LZW_uncompressInit (TLZW *lzw)
{
  int i;

  // vymazani tabulky frazi (nastaveni na volne)
  memset (lzw->phrase, NOT_USED, LZW_TABLE_MAXLEN*sizeof (TLZW_phrase));

  // slovnik je inicializovan pro vsechny vstupni symboly
  for (i = 0; i <= 255; i++)
  {
    lzw->phrase [i].index = LZW_putPhrase (lzw->phrase, NONE_PHRASE, i);
  }
  lzw->codeCount=256;
  lzw->code = NOT_USED; //
  lzw->mode = 1;
}


static void LZW_compress (TLZW *lzw)
{
  int code; // remember
  int c, nextCode;

  long bytesRead, bytesWritten;
  int codeCount;
  int (*getChar) (void);
  int (*putChar) (int);

  TLZW_phrase *phrase;

  bytesRead = lzw->bytesRead;
  bytesWritten = lzw->bytesWritten;
  codeCount = lzw->codeCount;
  phrase = lzw->phrase;
  code = lzw->code;
  getChar = lzw->getChar;
  putChar = lzw->putChar;

  // inicizalizace slovniku, vyplneni prvnich 256 jednoduchych frazi
//  LZW_compressInit ();
  while (c = getChar (), c != EOF)
  {
    bytesRead++;

    nextCode = LZW_hashFn (code, c);

    while ( ((phrase [nextCode].prev != code) ||
               (phrase [nextCode].value != c)) && (nextCode != NOT_USED) )
    {
      nextCode = phrase [nextCode].next;
    }

    if (nextCode != NOT_USED)
    {
      code = nextCode;
    }
    else
    {
      phrase [LZW_putPhrase (phrase, code, c)].index = codeCount;

      codeCount++;

      LZW_putCode (phrase [code].index, putChar, bytesWritten); // LZW_phrase [code].index);

      if (codeCount == LZW_TABLE_MAXPHRASE)
      {
        LZW_compressInit (lzw);
        codeCount = lzw->codeCount;
      }

      code = LZW_cache [c];
    }
  }
  LZW_putCode (phrase [code].index, putChar, bytesWritten); // LZW_phrase [code].index);

  LZW_putCodeEnd (putChar, bytesWritten);

  lzw->bytesRead = bytesRead;
  lzw->bytesWritten = bytesWritten;
  lzw->codeCount = codeCount;
  lzw->code = code;
}


inline static void LZW_push (int c)
{
  LZW_stack [LZW_sp] = c;
  LZW_sp++;
  if (LZW_sp >= LZW_TABLE_MAXLEN)
  {
    CGDK_exit ("CGDK fatal error: LZW stack overflowed.\n", EXIT_FAILURE);
  }
}

inline static int LZW_pop (void)
{
  if (LZW_sp > 0)
  {
    LZW_sp--;
    return LZW_stack [LZW_sp];
  }
  else
    return STACK_EMPTY;
}



static void LZW_uncompress (TLZW *lzw)
{
  int c, inCode, temp;
  int code, finChar; // remember

  long bytesRead, bytesWritten;
  int codeCount;
  int (*getChar) (void);
  int (*putChar) (int);
  TLZW_phrase *phrase;

//  LZW_uncompressInit ();

  bytesRead = lzw->bytesRead;
  bytesWritten = lzw->bytesWritten;
  codeCount = lzw->codeCount;
  phrase = lzw->phrase;
  code = lzw->code;
  finChar = lzw->finChar;
  getChar = lzw->getChar;
  putChar = lzw->putChar;

  while (inCode = LZW_getCode (getChar, bytesRead), inCode != EOF)
  {
    temp = phrase [inCode].index;

    if (temp != NOT_USED)
    {
      inCode = temp;

      while (phrase [inCode].prev != NONE_PHRASE)
      {
        LZW_push (phrase [inCode].value);
        inCode = phrase [inCode].prev;
      }

      finChar = phrase [inCode].value;

      putChar (finChar);

      if (code != NOT_USED)
      {
        phrase [codeCount].index = LZW_putPhrase (phrase, code, finChar);
        codeCount++;
      }
      code = temp;

      bytesWritten++;

      while (c = LZW_pop (), c != STACK_EMPTY)
      {
        putChar (c);
        bytesWritten++;
      }
    }
    else
    {
      temp = finChar;

      phrase [codeCount].index = LZW_putPhrase (phrase, code, temp);

      codeCount++;

      while (phrase [code].prev != NONE_PHRASE)
      {
        LZW_push (phrase [code].value);
        code = phrase [code].prev;
      }

      finChar = phrase [code].value;

      putChar (finChar);
      bytesWritten++;

      while (c = LZW_pop (), c != STACK_EMPTY)
      {
        putChar (c);
        bytesWritten++;
      }

      putChar (temp);
      bytesWritten++;

      code = phrase [inCode].index; // NOT_USED
    }

    if (codeCount == LZW_TABLE_MAXPHRASE)
    {
      LZW_uncompressInit (lzw);
      codeCount = lzw->codeCount;
    }
  }
  lzw->bytesRead = bytesRead;
  lzw->bytesWritten = bytesWritten;
  lzw->codeCount = codeCount;
  lzw->code = code;
  lzw->finChar = finChar;
}


void LZW_run (TLZW *lzw)
{
  if (lzw)
  {
    switch (lzw->mode)
    {
      case 0:
        LZW_compress (lzw);
        break;
      case 1:
        LZW_uncompress (lzw);
        break;
    }
  }
}

#include "lzwm_.cpp"
#include "lzwf_.cpp"
