
#include <stdlib.h>
#include <mem.h>

#include "svga.h"
#include "pal.h"

#include "paldef_.cpp"
#include "p256_.cpp"

int PAL_nearestColor (TRGBColor color, TRGBColor *pal, int palLen)
{
  int i;
  int minDif, dif;
  int nearest;

  minDif = 3*256;
  for (i = 0; i < palLen; i++)
  {
    dif = abs (color.r - pal [i].r);
    dif += abs (color.g - pal [i].g);
    dif += abs (color.b - pal [i].b);
    if (dif < minDif)
    {
      if (dif == 0)
      {
        return i;
      }
      else
      {
        nearest = i;
        minDif = dif;
      }
    }
  }
  return nearest;
}

int PAL_findColor (TRGBColor color, TRGBColor *pal, int palLen)
{
  int i;

  for (i = 0; i < palLen; i++)
  {
    if (*(long *) &pal [i] == *(long *) &color)
      return i;
  }
  return -1;
}


int PAL_insertColor (TRGBColor color, TRGBColor *pal, int palLen)
{
  int i;

  if (pal != NULL)
  {
    i = PAL_findColor (color, pal, palLen);

    if (i == -1)
    {
      i = 0;
      while ( (*((long *)pal+i) != 0) && (i < palLen))
        i++;

      if (i != palLen)
        pal [i] = color;
      else
      {
        return PAL_nearestColor (color, pal, palLen);
      }
      return i;
    }
    return i;
  }
  return 0;
}

int PAL_deleteColor (TRGBColor color, TRGBColor *pal, int palLen)
{
  int i;

  if (pal != NULL)
  {
    i = PAL_findColor (color, pal, palLen);

    if (i != -1)
    {
      *((long *) pal+i) = 0;
    }

    return i;
  }
  return 0;
}


int PAL_countColors (TRGBColor *pal, int palLen)
{
  int i;
  int count = 0;

  for (i = 0; i < palLen; i++)
  {
    if ( *((long *)pal+i) != 0)
      count++;
  }
  return count;
}


/*
static int PAL_findFreeBlock (int blockLen, TRGBColor *pal, int baseColor, int palLen)
{
  int i, j;


  for (i = 0; i < palLen-blockLen; i++)
  {
    for (j = i; j < i+blockLen; j++)
    {
      if ( *(long *) &pal [j+baseColor] != 0)
      {
        break;
      }
      return i+baseColor;
    }
  }
  return -1;
}
*/

void PAL_colorFadeStep (TRGBColor *newColor, TRGBColor startColor, TRGBColor endColor,
                        int stepCur, int stepMax)
{
  long delta;

  delta = endColor.r - startColor.r;
  delta = delta * stepCur / stepMax;
  newColor->r = startColor.r + delta;

  delta = endColor.g - startColor.g;
  delta = delta * stepCur / stepMax;
  newColor->g = startColor.g + delta;

  delta = endColor.b - startColor.b;
  delta = delta * stepCur / stepMax;
  newColor->b = startColor.b + delta;
}


void PAL_countFadeStep (TRGBColor *startPal, TRGBColor *endPal,
                       int stepCur, int stepMax, TRGBColor *fadePal,
                       int maxColor)
{
  int a, i;
  long delta;

  if ((startPal != NULL) && (endPal != NULL) && (fadePal != NULL))
  {
    for (i = 0; i < maxColor; i++)
    {
      PAL_colorFadeStep (fadePal+i,startPal [i], endPal [i], stepCur, stepMax);
    }
  }
}

void PAL_mix (TRGBColor *startPal, TRGBColor *endPal, int stepMax,
                   int baseColor, int maxColor)
{
  long step;
  TRGBColor fadePal [256];

  for (step = 0; step <= stepMax; step++)
  {
    PAL_countFadeStep (startPal+baseColor, endPal+baseColor, step, stepMax,
                       fadePal+baseColor, maxColor);
    SVGA_wait (1);
    PAL_set (fadePal, baseColor, maxColor);
  }
}

void PAL_fadeIn (TRGBColor *pal, int stepMax, int baseColor, int maxColor)
{
  TRGBColor startPal [256];

  memset (startPal, 0, sizeof startPal);
  PAL_mix (startPal, pal, stepMax, baseColor, maxColor);
}


void PAL_fadeOut (TRGBColor *pal, int stepMax, int baseColor, int maxColor)
{
  TRGBColor startPal [256];
  TRGBColor endPal [256];

  memset (endPal, 0, sizeof endPal);
  if (pal == NULL)
  {
    PAL_get (startPal, baseColor, maxColor);
    PAL_mix (startPal, endPal, stepMax, baseColor, maxColor);
  }
  else
  {
    PAL_mix (pal, endPal, stepMax, baseColor, maxColor);
  }
}

void PAL_fadeAway (TRGBColor *pal, int stepMax, int baseColor, int maxColor)
{
  TRGBColor startPal [256];
  TRGBColor endPal [256];

  memset (endPal, 255, sizeof endPal);

  if (pal == NULL)
  {
    PAL_get (startPal, baseColor, maxColor);
    PAL_mix (startPal, endPal, stepMax, baseColor, maxColor);
  }
  else
  {
    PAL_mix (pal, endPal, stepMax, baseColor, maxColor);
  }
}

void PAL_rotateGreen (TRGBColor *pal, int count)
{

}

void PAL_rotateRed (TRGBColor *pal, int count)
{
  int i;
  unsigned char c;

  for (i = 0; i < count; i++)
  {
    c = pal [i].r;
    pal [i].r = pal [i].g;
    pal [i].g = pal [i].b;
    pal [i].b = c;
  }
}

void PAL_rotateBlue (TRGBColor *pal, int count)
{
  int i;
  unsigned char c;

  for (i = 0; i < count; i++)
  {
    c = pal [i].b;
    pal [i].b = pal [i].g;
    pal [i].g = pal [i].r;
    pal [i].r = c;
  }
}

void PAL_mirrorRed (TRGBColor *pal, int count)
{
  int i;
  unsigned char c;

  for (i = 0; i < count; i++)
  {
    c = pal [i].b;
    pal [i].b = pal [i].g;
    pal [i].g = c;
  }
}

void PAL_mirrorGreen (TRGBColor *pal, int count)
{
  int i;
  unsigned char c;

  for (i = 0; i < count; i++)
  {
    c = pal [i].r;
    pal [i].r = pal [i].b;
    pal [i].b = c;
  }
}

void PAL_mirrorBlue (TRGBColor *pal, int count)
{
  int i;
  unsigned char c;

  for (i = 0; i < count; i++)
  {
    c = pal [i].g;
    pal [i].r = pal [i].g;
    pal [i].g = c;
  }
}




void PAL_setDefault (void)
{
  SVGA_wait (1);
  PAL_set (PAL_default, 0, 256);
}


void PAL_PAL2RGB (TRGBColor *RGBPAL, char *Pal, int maxColor)
{
  int i,j;

  j = 0;
  for (i=0; i<maxColor; i++)
  {
    RGBPAL[i].r=4*Pal [j];
    j++;
    RGBPAL[i].g=4*Pal [j];
    j++;
    RGBPAL[i].b=4*Pal [j];
    j++;
    RGBPAL[i].a=0;
  }
}
