
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <mem.h>

#include "cgdk.h"
#include "svga.h"
#include "pal.h"
#include "keyb.h"
#include "pcx.h"
#include "car.h"
#include "tm.h"
#include "file.h"
#include "scroll.h"
#include "mouse.h"
#include "timer.h"

#ifdef __TURBOC__
  #define tileX 8
  #define tileY 8
#else
  #define tileX 48
  #define tileY 48
#endif

#define PCXVIEW_ERR_NOTFOUND 1
#define PCXVIEW_ERR_SETVIDEOMODE 2
#define PCXVIEW_ERR_LOAD 3
#define PCXVIEW_ERR_MEMORY 4

char *pcxView_fileName;

static int pcxView (TCAR *carHandle, TF_list *files, int videoMode)
{
  int sizeX,sizeY;
  int limX, limY;
  int bytesPerPixel;
  char *pcxBuf,*sprBuf;
  short *mapBuf;
  int mapLenX,mapLenY;
  TRGBColor pal [256];
  int mode=videoMode;

  KEYB_init ();
  pcxView_fileName = F_listNext (files);

  do
  {
    if (PCX_getInfo (carHandle, pcxView_fileName, &sizeX, &sizeY, &bytesPerPixel))
    {
      return PCXVIEW_ERR_NOTFOUND;
    }

    if (videoMode == -1)
    {
      mode = SVGA_recommendVideoMode (sizeX, sizeY, 1, VMODE_ANY);
    }

    if (mode != -1)
    {
      if (SVGA_setVideoMode (mode) != 0)
      {
        return PCXVIEW_ERR_SETVIDEOMODE;
      }
      PAL_clear (0, 256);
      mode = -1;
    }

    pcxBuf = (char *) _alloc ((unsigned)(sizeX * sizeY));

    if (PCX_load (carHandle, pcxView_fileName, pcxBuf, sizeX, sizeY, pal))
    {
      return PCXVIEW_ERR_LOAD;
    }

    sprBuf = CSF_createSprBuffer (pcxBuf, sizeX, sizeY, tileX, tileY);
    if (!sprBuf)
    {
      return PCXVIEW_ERR_MEMORY;
    }

    mapBuf = CSF_createMapBuffer (sizeX, sizeY, tileX, tileY, &mapLenX, &mapLenY);
    if (!mapBuf)
    {
      return PCXVIEW_ERR_MEMORY;
    }
    _free (pcxBuf);

    if (SVGA_maxX >= sizeX)
      limX = 0;
    else
      limX = sizeX-(SVGA_maxX-1);

    if (SVGA_maxY >= sizeY)
      limY = 0;
    else
      limY = sizeY-(SVGA_maxY-1);

    SCROLL_setMap (mapBuf, mapLenX, mapLenY);
    SCROLL_setSpr (sprBuf, tileX, tileY);
    SCROLL_setLimit (limX, limY, SCROLL_NOLIMIT, SCROLL_NOLIMIT);
    PAL_clear (0, 256);
    SCROLL_redraw (0,0, -1, -1);
    PAL_fadeIn (pal, 100, 0, 256);

    switch (SCROLL_walk ())
    {
      case SCROLL_WALK_EXIT:
      {
        pcxView_fileName = NULL;
        break;
      }
      case SCROLL_WALK_NEXT:
      {
        pcxView_fileName = F_listNext (files);
        break;
      }
      case SCROLL_WALK_PREV:
      {
        pcxView_fileName = F_listPrev (files);
        break;
      }
    }
    _free (mapBuf);
    _free (sprBuf);
  }
  while (pcxView_fileName != NULL);

  return 0;
}

int main (void)
{
  int videoMode;
  int exitCode;
  TF_list *files;
  TCAR *carHandle;

  printf ("\n"
          "PCX - Easy! PCX picture viewer\n"
          "\n%s\n",CGDK_copyright);

  CGDK_init ();
  TIMER_init (TIMER_OLDFREQ_DEF);

  switch (_argc)
  {
    case 3:
    {
      videoMode = strtol (_argv [2], NULL, 0);
      break;
    }
    case 2:
    {
      videoMode = -1;
      break;
    }
    default:
    {
      CGDK_exit ("Syntax is : PCX <filename.pcx> [video_mode]\n", EXIT_FAILURE);
    }
  }

  files = F_listNew (_argv [1]);
  printf ("Found %i files matching \"%s\"\n",files->max,_argv [1]);
  if (files->max == 0)
  {
    CGDK_exit ("Nothing to view\n", EXIT_FAILURE);
  }

  if ((SVGA_compatible & VGA_COMPATIBLE) == 0)
  {
    CGDK_exit ("This program requires a VGA compatible videocard.\n", EXIT_FAILURE);
  }

  carHandle = CAR_open (CAR_EMULATION);

  exitCode = pcxView (carHandle, files, videoMode);

  CAR_close (carHandle);

  switch (exitCode)
  {
    case PCXVIEW_ERR_SETVIDEOMODE:
    {
      CGDK_exit ("The specified video mode is not supported.\n", EXIT_FAILURE);
      break;
    }
    case PCXVIEW_ERR_MEMORY:
    {
      CGDK_exit ("\nNot enough memory.\n", EXIT_FAILURE);
      break;
    }
    case PCXVIEW_ERR_NOTFOUND:
    {
      sprintf (exitStr, "\"%s\" is an invalid PCX file.\n", pcxView_fileName);
      CGDK_exit (exitStr, EXIT_FAILURE);
      break;
    }
    case PCXVIEW_ERR_LOAD:
    {
      sprintf (exitStr, "Error loading \"%s\".\n", pcxView_fileName);
      CGDK_exit (exitStr, EXIT_FAILURE);
      break;
    }
  }

  F_listDelete (files);

  return EXIT_SUCCESS;
}
