
#include <stdlib.h>
#include "cgdk.h"

#include "stars.h"

struct STAR
{
  long x,y,z;
  long lastPos;
};

static long maxStars;              // maximum count of stars
static long activeStars;           // current count of active stars
static long distance;
static long zoom;
static long screenX,screenY;
static long warpSpeed;             // the speed
static long firstColor;
static long numColors;

static struct STAR *stars;         // pointer to stars structure
static void far *screenBuf = NULL;


static void createStar (void)
{
  long i = 0;

  if (stars != NULL) //&& (ActiveStars < MaxStars))
  {
    for (i = 0; (i < maxStars) && (stars [i].z > 0); i++)
      ;
    if (i < maxStars)
    {
      activeStars++;
      stars [i].lastPos = -1;

      stars [i].x = (distance >> 1) - (rand () % distance);
      stars [i].y = (distance >> 1) - (rand () % distance);
      stars [i].z = distance;
    }
  }
}

static void displayStars (void)
{
  long i;
  unsigned long posX,posY,di;
  long newZ;

  for (i = 0; i < maxStars; i++)
  {
    if (stars [i].z > 0)
    {
      if (stars [i].lastPos != -1)
        ((char far *) screenBuf) [stars [i].lastPos] = 0;

      if (
      (((posY = (screenY>>1) + stars [i].y*zoom/stars[i].z) >= screenY))
                  ||
      (((posX = (screenX>>1) + stars [i].x*zoom/stars[i].z) >= screenX))
                  ||
      (stars [i].z - warpSpeed) < 0)
        stars [i].z = 0;
      else
      {
        di=posY*screenX+posX;
        if (((char far *)screenBuf) [di] == 0)
        {
          ((char far *)screenBuf) [di] = firstColor + numColors - (numColors * stars [i].z / distance);
          stars [i].lastPos = di;
        }
        else
          stars [i].lastPos = -1;
        stars [i].z -= warpSpeed;
      }
    }
  }
}

void STARS_done (void)
{
  if (stars != NULL)
  {
    _free (stars);
    stars = NULL;
  }
}



int STARS_init (void far *newScreenBuf, int newScreenX, int newScreenY, int newMaxStars)
{
  STARS_done ();
  _atexit (STARS_done);
  // alocate memory for all desired stars
  maxStars = newMaxStars;
  screenBuf = newScreenBuf;
  screenX = newScreenX;
  screenY = newScreenY;
  stars = (STAR *) _calloc (sizeof (STAR)*newMaxStars, 1, NULL);
  if (stars == NULL)
    return 1;
  return 0;
}

void STARS_setSpeed (int newWarpSpeed)
{
  warpSpeed = newWarpSpeed;
}

void STARS_setDistance (int newDistance)
{
  distance = newDistance;
}

void STARS_setZoom (int newZoom)
{
  zoom = newZoom;
}

void STARS_setColors (int newFirstColor, int newNumColors)
{
  firstColor = newFirstColor;
  numColors = newNumColors-1;
}



void STARS_update (void)
{
  if (stars != NULL)
  {
    createStar ();
    createStar ();
    createStar ();
    displayStars ();
  }
}
