
#include <math.h>

#include "timer.h"
#include "svga.h"
#include "spr.h"
#include "keyb.h"
#include "lin.h"
#include "cgdk.h"
#include "dpmi.h"

#define PUTSPRNOVSB 1
#define PUTWINNOVSB 2
#define PUTSPR 3
#define PUTWIN 4

struct TSVGAC
{
  void *spr;
  short x;
  short  y;
  short sizeX;
  short sizeY;
  short formatX;
  short type;
};

static TSVGAC *SVGAC;
static int SVGAC_last;
int SVGAC_maxCached;

void SVGAC_done (void)
{
  if (SVGAC)
  {
    _free (SVGAC);
    SVGAC = NULL;
  }
  SVGAC_maxCached = 0;
  SVGAC_last = 0;
}

void SVGAC_init (int maxCachedSprites)
{
  SVGAC_done ();
  SVGAC_maxCached = maxCachedSprites;
  SVGAC = (TSVGAC *) _alloc (maxCachedSprites * sizeof (TSVGAC));
}

static int SVGAC_insert (void *Sprite, int x, int y, int sizeX, int sizeY, int formatX, int type)
{
  int i;
  TSVGAC *svgac;

  if (SVGAC_last < SVGAC_maxCached)
  {
    svgac=SVGAC+SVGAC_last;
    svgac->spr = Sprite;
    svgac->x = x;
    svgac->y = y;
    svgac->sizeX = sizeX;
    svgac->sizeY = sizeY;
    svgac->formatX = formatX;
    svgac->type = type;
    SVGAC_last++;
    return 0;
  }
  else
  {
    return 1;
  }
}

void SVGAC_putSprNoVSB (void *Sprite, int x, int y, int sizeX, int sizeY, int formatX)
{
  int i;

  if (SVGAC_insert (Sprite, x, y, sizeX, sizeY, formatX, PUTSPRNOVSB) != 0)
  {
    SVGA_putSprNoVSB (Sprite, x, y, sizeX, sizeY, formatX);
  }
}

void SVGAC_putWinNoVSB (void *Sprite, int x, int y, int sizeX, int sizeY, int formatX)
{
  int i;

  if (SVGAC_insert (Sprite, x, y, sizeX, sizeY, formatX, PUTWINNOVSB) != 0)
  {
    SVGA_putWinNoVSB (Sprite, x, y, sizeX, sizeY, formatX);
  }
}

void SVGAC_putSpr (void *Sprite, int x, int y, int sizeX, int sizeY, int formatX)
{
  int i;

  if (SVGAC_insert (Sprite, x, y, sizeX, sizeY, formatX, PUTSPR) != 0)
  {
    SVGA_putSpr (Sprite, x, y, sizeX, sizeY, formatX);
  }
}

void SVGAC_putWin (void *Sprite, int x, int y, int sizeX, int sizeY, int formatX)
{
  int i;

  if (SVGAC_insert (Sprite, x, y, sizeX, sizeY, formatX, PUTWIN) != 0)
  {
    SVGA_putWin (Sprite, x, y, sizeX, sizeY, formatX);
  }
}



static void SVGAC_insertSort (void)
{
  int i,j;
  TSVGAC tsvgac;
  TSVGAC *svgac, *svgac2;
  int sx1, sx2, sy1, sy2;
  int dx1, dx2, dy1, dy2;

  for (i = 0; i < SVGAC_last; i++)
  {
    svgac = SVGAC+i;
    sy1 = svgac->y;
    sy2 = sy1+svgac->sizeY;
    sx1 = svgac->x;
    sx2 = sx1+svgac->sizeX;

    for (j = i; j > 0; j--)
    {
      svgac2 = svgac;
      svgac--;
      dy1 = svgac->y;
      if (sy1 >= dy1)
        break;
      dy2 = dy1+svgac->sizeY;
      dx1 = svgac->x;
      dx2 = dx1+svgac->sizeX;
      if (SPR_touch (sx1, sy1, sx2, sy2, dx1, dy1, dx2, dy2))
        break;

      tsvgac = *svgac2;
      *svgac2 = *svgac;
      *svgac = tsvgac;
    }
  }
}


static void SVGAC_put (TSVGAC *svgac)
{
  switch (svgac->type)
  {
    case PUTSPRNOVSB:
    {
      SVGA_putSprNoVSB (svgac->spr, svgac->x, svgac->y,
                        svgac->sizeX, svgac->sizeY, svgac->formatX);
      break;
    }
    case PUTWINNOVSB:
    {
      SVGA_putWinNoVSB (svgac->spr, svgac->x, svgac->y,
                        svgac->sizeX, svgac->sizeY, svgac->formatX);
      break;
    }
    case PUTSPR:
    {
      SVGA_putSpr (svgac->spr, svgac->x, svgac->y,
                   svgac->sizeX, svgac->sizeY, svgac->formatX);
      break;
    }
    case PUTWIN:
    {
      SVGA_putWin (svgac->spr, svgac->x, svgac->y,
                   svgac->sizeX, svgac->sizeY, svgac->formatX);
      break;
    }

  }
}




void SVGAC_flush (void)
{
  int i;
  TSVGAC *svgac;
  long beamY;
  long beamMaxY;
  long end1Y, end2Y;


//  if (KEYB_pressed (KEY_SPACE))
//  {
//    DPMI_debugInt ();
//  }

  SVGAC_insertSort ();

  svgac = SVGAC;

  #ifndef CGDK_TRIAL
  if (SVGA_getWin_ == B256_getWin)
  {
    for (i = 0; i < SVGAC_last; i++)
    {
      SVGAC_put (SVGAC+i);
      *(int *) &svgac += sizeof (TSVGAC);
    }

    B256C_flush ();
  }
  else
  #endif
  {
    i = 0;

    if (SVGA_enableRetrace)
    {
      beamY = /*SVGA_curYPrev +*/ SVGA_maxY/3;
      beamMaxY = /*SVGA_curYPrev +*/ SVGA_maxY;

      while (fmod (TIMER_retraceTime, TIMER_retraceTimeMaxF) < 0.666*TIMER_retraceTimeMaxF)
	;

      while (i < SVGAC_last)
      {
        end1Y =  svgac->y + svgac->sizeY;
  //      end2Y = end1Y + SVGA_virtualY;

        if ( ((end1Y < beamY) || (end1Y > beamMaxY)) )//&&
  //             ((!SVGA_doubleBuffer) || (end2Y < beamY) || (end2Y > beamMaxY)))
        {
          SVGAC_put (svgac);
          *(int *) &svgac += sizeof (TSVGAC);
          i++;
        }
        else
        {
          break;
        }
      }
    }

    SVGA_wait (1);

    for ( ; i < SVGAC_last; i++)
    {
      SVGAC_put (svgac);
      *(int *) &svgac += sizeof (TSVGAC);
    }
  }

  SVGAC_last = 0;
}
